@@ -1,21 +1,24 @@
-{-# LANGUAGE CPP, OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Bodhi (
   bodhiCreateOverride,
-  bodhiTestingRepo,
+  bodhiTestingRepoTag,
   checkAutoBodhiUpdate,
   UpdateType(..),
   UpdateSeverity(..),
-  bodhiUpdate
+  UpdateNotes(..),
+  bodhiUpdate,
+  bodhiBuildExists
   )
 where
 
-#if MIN_VERSION_aeson(2,0,0)
-import Data.Aeson.Key (fromText)
-#endif
-import Data.Aeson.Types (Object, (.:), parseEither)
+import Data.Aeson.Types (Object)
 import Data.Char (isDigit)
+import Data.RPM.NVR (NVR)
+import Distribution.Fedora.Branch (branchRelease)
+import Distribution.Fedora.Release (Release(..))
 import Fedora.Bodhi hiding (bodhiUpdate)
+import SimplePrompt (promptEnter, promptNonEmpty)
 import Text.Read
 import qualified Text.ParserCombinators.ReadP as R
 import qualified Text.ParserCombinators.ReadPrec as RP
@@ -24,46 +27,30 @@ import Branches
 import Bugzilla (BugId)
 import Common
 import Common.System
-import qualified Common.Text as T
+import Git (gitSwitchBranch)
 import Package
-import Prompt
+import Types (ChangeType(ChangeBodhi))
 
 checkAutoBodhiUpdate :: Branch -> IO Bool
 checkAutoBodhiUpdate Rawhide = return True
--- epel7 returns 'create_automatic_updates: null' !
-checkAutoBodhiUpdate (EPEL 7) = return False
--- not sure how to best handle next
-checkAutoBodhiUpdate (EPELNext _) = return False
 checkAutoBodhiUpdate br =
-  lookupKey'' "create_automatic_updates" <$> bodhiRelease (show br)
-  where
-    -- Error in $: key "create_automatic_updates" not found
-    lookupKey'' :: T.Text -> Object -> Bool
-    lookupKey'' k obj =
-      let errMsg e = error $ e ++ " " ++ show obj in
-        -- bodhi-hs has lookupKeyEither
-        either errMsg id $ parseEither (.: fromText k) obj
-
-#if !MIN_VERSION_aeson(2,0,0)
-    fromText :: T.Text -> T.Text
-    fromText = id
-#endif
+  releaseAutomaticUpdates <$> branchRelease br
 
 -- FIXME should determine 3 days for branched devel release
 -- FIXME handle expired override?
-bodhiCreateOverride :: Bool -> Maybe Int -> String -> IO ()
+bodhiCreateOverride :: Bool -> Maybe Int -> NVR -> IO ()
 bodhiCreateOverride dryrun mduration nvr = do
-  putStrLn $ "Creating Bodhi Override for " ++ nvr ++ ":"
+  putStrLn $ "Creating Bodhi Override for" +-+ showNVR nvr ++ ":"
   unless dryrun $ do
-    ok <- cmdBool "bodhi" ["overrides", "save", "--notes", "chain building with fbrnch", "--duration", show (fromMaybe 4 mduration), "--no-wait", nvr]
+    ok <- cmdBool "bodhi" ["overrides", "save", "--notes", "chain building with fbrnch", "--duration", show (fromMaybe 4 mduration), "--no-wait", showNVR nvr]
     if ok
-      then putStrLn $ "https://bodhi.fedoraproject.org/overrides/" ++ nvr
+      then putStrLn $ "https://bodhi.fedoraproject.org/overrides/" ++ showNVR nvr
       else do
-      moverride <- bodhiOverride nvr
+      moverride <- bodhiOverride $ showNVR nvr
       case moverride of
         Nothing -> do
           putStrLn "bodhi override failed"
-          prompt_ "Press Enter to retry"
+          promptEnter "Press Enter to retry"
           bodhiCreateOverride dryrun mduration nvr
         -- FIXME prettyprint
         Just obj -> error' $ show obj
@@ -113,44 +100,52 @@ instance Read UpdateSeverity where
       "urgent" -> RP.lift (R.string s) >> return SeverityUrgent
       _ -> error' "unknown bodhi update severity" >> RP.pfail
 
-bodhiTestingRepo :: Branch -> IO (Maybe String)
-bodhiTestingRepo Rawhide = return Nothing
-bodhiTestingRepo br = do
-  obj <- bodhiRelease (show br)
-  return $
-    case lookupKey "testing_repository" obj :: Maybe String of
-      Nothing -> Nothing
-      Just _ -> lookupKey' "testing_tag" obj
+bodhiTestingRepoTag :: Branch -> IO (Maybe String)
+bodhiTestingRepoTag Rawhide = return Nothing
+bodhiTestingRepoTag br = do
+  rel <- branchRelease br
+  return $ do
+    _ <- releaseTestingRepo rel
+    pure $ releaseTestingTag rel
 
+data UpdateNotes = NotesChangelog | NotesText String
+
+-- FIXME support --no-close-bugs
 -- push comma separated list of builds for a package to bodhi
 bodhiUpdate :: Bool -> (Maybe UpdateType, UpdateSeverity) -> Maybe BugId
-            -> Bool -> FilePath -> String -> IO ()
-bodhiUpdate _ _ _ _ _ [] = error' "cannot make empty update"
-bodhiUpdate dryrun (mupdate,severity) mreview usechangelog spec nvrs = do
+            -> Maybe UpdateNotes -> Maybe Branch -> FilePath -> String -> IO ()
+bodhiUpdate _ _ _ _ _ _ [] = putStrLn "no package to push"
+bodhiUpdate dryrun (mupdate,severity) mreview mnotes mrbr spec nvrs = do
   case mupdate of
     Nothing -> return ()
     Just updateType ->
       unless dryrun $ do
-        -- use cmdLog to debug, but notes are not quoted
-        updatedone <- do
+        updateDone <- do
           mtemplate <- maybeTemplate updateType
           case mtemplate of
             Just file -> do
+              -- FIXME use cmdLog_ to debug, but notes are not quoted
               cmd_ "bodhi" ["updates", "new", "--file", file, nvrs]
               return True
             Nothing -> do
+              whenJust mrbr $ gitSwitchBranch . RelBranch
               -- FIXME also query for open existing bugs
               changelog <- if isJust mreview
                            then getSummaryURL spec
                            else
-                             if usechangelog
-                             then cleanChangelog spec
-                             else
-                               -- FIXME list open bugs
-                               changeLogPrompt (Just "update") spec
+                             case mnotes of
+                               Just NotesChangelog ->
+                                 cleanChangelog True spec
+                               Just (NotesText notes) ->
+                                 return notes
+                               Nothing ->
+                                 -- FIXME list open bugs
+                                 changeLogPrompt ChangeBodhi spec
               if trim (lower changelog) `elem` ["no","n"]
                 then return False
                 else do
+                when (length changelog > 10000) $
+                  putStrLn "Bodhi only accepts up to 10000 chars: will be truncated"
                 let cbugs = extractBugReferences changelog
                     bugs =
                       let bids = [show rev | Just rev <- [mreview]] ++ cbugs in
@@ -160,30 +155,35 @@ bodhiUpdate dryrun (mupdate,severity) mreview usechangelog spec nvrs = do
                 when (isJust mreview &&
                       updateType `elem` [SecurityUpdate,BugfixUpdate]) $
                   warning "overriding update type with 'newpackage'"
-                putStrLn $ "Creating Bodhi Update for " ++ nvrs ++ ":"
+                putStrLn $ "Creating Bodhi Update for" +-+ nvrs ++ ":"
                 -- FIXME check for Bodhi URL to confirm update
-                cmd_ "bodhi" $ ["updates", "new", "--type", if isJust mreview then "newpackage" else show updateType, "--severity", show severity, "--request", "testing", "--notes", changelog, "--autokarma", "--autotime", "--close-bugs"] ++ bugs ++ [nvrs]
+                -- FIXME returns json error string if it exists:
+                -- {"status": "error", "errors": [{"location": "body", "name": "builds", "description": "Update for ghc9.2-9.2.5-14.fc36 already exists"}]}
+                cmd_ "bodhi" $ ["updates", "new", "--type", if isJust mreview then "newpackage" else show updateType, "--severity", show severity, "--request", "testing", "--notes", take 10000 changelog, "--autokarma", "--autotime", "--close-bugs"] ++ bugs ++ [nvrs]
                 return True
-        when updatedone $ do
+        when updateDone $ do
           -- FIXME avoid this if we know the update URLs (split update does not seem to return URLs)
-          updates <- bodhiUpdates [makeItem "display_user" "0", makeItem "builds" nvrs]
-          if null updates
-            then do
-            putStrLn $ "bodhi submission failed for " ++ nvrs
-            prompt_ "Press Enter to resubmit to Bodhi"
-            bodhiUpdate dryrun (mupdate,severity) mreview usechangelog spec nvrs
-            else
-            forM_ updates $ \update ->
-            case lookupKey "url" update of
-              Nothing -> error' "Update created but no url"
-              Just uri -> putStrLn uri
+          eupdates <- bodhiUpdatesEither [makeItem "display_user" "0", makeItem "builds" nvrs]
+          case eupdates of
+            Left err -> error' err
+            Right updates ->
+              if null updates
+              then do
+              putStrLn $ "bodhi submission failed for" +-+ nvrs
+              promptEnter "Press Enter to resubmit to Bodhi"
+              bodhiUpdate dryrun (mupdate,severity) mreview mnotes mrbr spec nvrs
+              else
+              forM_ updates $ \update ->
+              case lookupKey "url" update of
+                Nothing -> error' "Update created but no url"
+                Just uri -> putStrLn uri
   where
     extractBugReferences :: String -> [String]
     extractBugReferences clog =
       case dropWhile (/= '#') clog of
         "" -> []
-        rest ->
-          case span isDigit (tail rest) of
+        (_:rest) ->
+          case span isDigit rest of
             (ds,more) ->
               -- make sure is contemporary 7-digit bug
               (if length ds > 6 then (ds :) else id) $
@@ -191,11 +191,24 @@ bodhiUpdate dryrun (mupdate,severity) mreview usechangelog spec nvrs = do
 
     maybeTemplate :: UpdateType -> IO (Maybe FilePath)
     maybeTemplate TemplateUpdate = do
-      file <- prompt "Please input the update template filepath"
+      file <- promptNonEmpty "Please input the update template filepath"
       exists <- doesFileExist file
       if exists
         then return $ Just file
         else do
-        putStrLn ("no such file: " ++ file)
+        putStrLn ("no such file:" +-+ file)
         maybeTemplate TemplateUpdate
     maybeTemplate _ = return Nothing
+
+bodhiBuildExists :: NVR -> IO Bool
+bodhiBuildExists nvr = do
+  obj <- bodhiBuild $ showNVR nvr
+  return $ isNothing (lookupKey "status" obj :: Maybe String)
+
+bodhiUpdatesEither :: Query -> IO (Either String [Object])
+bodhiUpdatesEither params = do
+  res <- queryBodhi params "updates/"
+  return $
+    case lookupKey "updates" res of
+      Just updates -> Right updates
+      Nothing -> Left $ "failed to read updates:\n" ++ show res
