@@ -38,6 +38,7 @@ module Bugzilla (
   statusNewPost,
   statusNewModified,
   statusOpen,
+  statusRelPrep,
   summaryContains,
   versionIs,
   ftbfsFedoraBugs,
@@ -66,18 +67,20 @@ module Bugzilla (
   putBugURLStatus,
   -- request
   newBzRequest,
-  makeTextItem
+  makeTextItem,
+  searchUsers
   ) where
 
 import Common
 import Common.System
 import qualified Common.Text as T
-import Prompt
+import SimplePrompt (promptNonEmpty)
 
 import Data.Aeson.Types (Array, Object)
 import qualified Data.ByteString.Char8 as B
 import Data.ByteString.UTF8
 import Data.Ini.Config
+import Data.RPM.NVR (NVR)
 import Network.HTTP.Conduit
 import Network.HTTP.Query
 import Network.HTTP.Simple
@@ -108,7 +111,7 @@ commentBug session bid comment = do
     case lookupKey "message" res of
       Nothing -> print res
       Just msg -> T.putStrLn msg
-    error' $ "failed to update bug " ++ show bid
+    error' $ "failed to update bug" +-+ show bid
   putStrLn "Comment added"
 
 updateBug :: BugzillaSession -> BugId -> [(String,String)] -> IO ()
@@ -125,7 +128,7 @@ updateBug session bid params = do
     case lookupKey "message" res of
       Nothing -> print res
       Just msg -> T.putStrLn msg
-    error' $ "failed to update bug " ++ show bid
+    error' $ "failed to update bug" +-+ show bid
 
 encodeParams :: [(String, String)] -> [(ByteString, ByteString)]
 encodeParams [] = []
@@ -133,12 +136,12 @@ encodeParams ((k,v):ps) =
   (B.pack k, fromString v) : encodeParams ps
 
 -- FIXME check original status?
-putBugBuild :: Bool -> BugzillaSession -> BugId -> String -> IO ()
+putBugBuild :: Bool -> BugzillaSession -> BugId -> NVR -> IO ()
 putBugBuild dryrun session bid nvr = do
   unless dryrun $
     void $ updateBug session bid
-    [("cf_fixed_in", nvr), ("status", "MODIFIED")]
-  putStrLn $ "bug " ++ show bid ++ (if dryrun then " would be" else "") ++ " moved to MODIFIED with " ++ nvr
+    [("cf_fixed_in", showNVR nvr), ("status", "MODIFIED")]
+  putStrLn $ "bug" +-+ show bid ++ (if dryrun then " would be" else "") +-+ "moved to MODIFIED with" +-+ showNVR nvr
 
 brc :: T.Text
 brc = "bugzilla.redhat.com"
@@ -185,13 +188,13 @@ getBzUser = do
     then readIniConfig config rcParser rcUserEmail
     else do
     -- FIXME: option to override email
-    email <- prompt "Bugzilla Username"
+    email <- promptNonEmpty "Bugzilla Username"
     when (emailIsValid email) $ do
       let configDir = takeDirectory config
       configDirExists <- doesDirectoryExist configDir
       unless configDirExists $ createDirectory configDir
       T.writeFile config $ "[" <> brc <> "]\nuser = " <> T.pack email <> "\n"
-      putStrLn $ "Saved in " ++ config ++ "\n"
+      putStrLn $ "Saved in" +-+ config ++ "\n"
     getBzUser
   where
     rcParser :: IniParser BzUserRC
@@ -232,7 +235,7 @@ bzApiKeySession = do
      "Create a key at https://bugzilla.redhat.com/userprefs.cgi?tab=apikey",
      "Save the key under in '" ++ config ++ "' under:",
      "[bugzilla.redhat.com]",
-     apiKeyField ++ " = <YourApiKey>"]
+     apiKeyField +-+ "= <YourApiKey>"]
     else do
     apikey <- readIniConfig config rcParser bzApiKey
     return $ ApiKeySession brc $ BugzillaApiKey apikey
@@ -276,18 +279,22 @@ statusNewModified :: SearchExpression
 statusNewModified =
   StatusField `equalsAny` ["NEW", "ASSIGNED", "POST", "MODIFIED"]
 
+statusRelPrep :: SearchExpression
+statusRelPrep =
+  StatusField .==. "RELEASE_PENDING"
+
 reviewApproved :: SearchExpression
 reviewApproved =
   FlagsField `contains` "fedora-review+"
 
 pkgReviews :: String -> SearchExpression
 pkgReviews pkg =
-  SummaryField `contains` T.pack ("Review Request: " ++ pkg ++ " - ") .&&.
+  SummaryField `contains` T.pack ("Review Request:" +-+ pkg +-+ "- ") .&&.
   packageReview
 
 pkgReviewsPrefix :: String -> SearchExpression
 pkgReviewsPrefix prefix =
-  SummaryField `contains` T.pack ("Review Request: " ++ prefix) .&&.
+  SummaryField `contains` T.pack ("Review Request:" +-+ prefix) .&&.
   packageReview
 
 pkgBugs :: String -> SearchExpression
@@ -326,7 +333,7 @@ reviewBugIdSession :: String -> IO (BugId,BugzillaSession)
 reviewBugIdSession pkg = do
   bugs <- bugIdsAnon $ pkgReviews pkg .&&. statusOpen
   case bugs of
-    [] -> error $ "No review bug found for " ++ pkg
+    [] -> error' $ "No review bug found for" +-+ pkg
     [bug] -> do
       session <- bzApiKeySession
       return (bug, session)
@@ -337,7 +344,7 @@ approvedReviewBugIdSession pkg = do
   bugs <- bugIdsAnon $
           pkgReviews pkg .&&. statusOpen .&&. reviewApproved
   case bugs of
-    [] -> error $ "No review bug found for " ++ pkg
+    [] -> error' $ "No review bug found for" +-+ pkg
     [bug] -> do
       session <- bzApiKeySession
       return (bug, session)
@@ -348,15 +355,17 @@ approvedReviewBugSession pkg = do
   bugs <- bugsAnon $
           pkgReviews pkg .&&. statusOpen .&&. reviewApproved
   case bugs of
-    [] -> error $ "No review bug found for " ++ pkg
+    [] -> error' $ "No review bug found for" +-+ pkg
     [bug] -> do
       session <- bzApiKeySession
       return (bug, session)
     _ -> error' "more than one review bug found!"
 
 reviewBugToPackage :: Bug -> String
-reviewBugToPackage =
-  head . dropOne ["Request:"] . dropOne ["Review", "Re-Review", "Rename"]  . words . T.unpack . bugSummary
+reviewBugToPackage bug =
+  case (dropOne ["Request:"] . dropOne ["Review", "Re-Review", "Rename"]  . words . T.unpack . bugSummary) bug of
+    [] -> error' $ "failed to determine review package for" +-+ show (bugId bug)
+    (p:_) -> p
   where
     dropOne _ [] = []
     dropOne ks as@(w:ws) = if w `elem` ks then ws else as
@@ -366,7 +375,7 @@ readIniConfig inifile iniparser fn = do
   ini <- T.readFile inifile
   return $
     case parseIniFile ini iniparser of
-      Left err -> error' $ err ++ "\nin " ++ inifile
+      Left err -> error' $ err ++ "\nin" +-+ inifile
       Right res -> fn res
 
 sortBugsByID :: [Bug] -> [Bug]
@@ -399,10 +408,11 @@ showComment cmt = do
   mapM_ (T.putStrLn . ("  " <>)) $ dropDuplicates . removeLeadingNewline . T.lines $ commentText cmt
   putNewLn
 
+-- FIXME check for Fedora Admin user for bugzilla script actions <fedora-admin-xmlrpc@fedoraproject.org>
 checkRepoCreatedComment :: BugzillaSession -> BugId -> IO Bool
 checkRepoCreatedComment session bid =
     checkForComment session bid
-      "(fedscm-admin):  The Pagure repository was created at"
+      "The Pagure repository was created at https://src.fedoraproject.org/"
 
 checkForComment :: BugzillaSession -> BugId -> T.Text -> IO Bool
 checkForComment session bid text = do
@@ -414,7 +424,7 @@ putReviewBug short bug = do
   if short
     then putStr $ reviewBugToPackage bug ++ " "
     else do
-    putStr $ T.unpack (bugStatus bug) ++ " " ++ reviewBugToPackage bug ++ ": "
+    putStr $ T.unpack (bugStatus bug) +-+ reviewBugToPackage bug ++ ": "
     putBugId $ bugId bug
 
 putBug :: Bug -> IO ()
@@ -425,18 +435,20 @@ putBug bug = do
 
 putBugVer :: Bug -> IO ()
 putBugVer bug = do
-  T.putStr $ "[" <> prodVersion <> "] "
+  T.putStr $ "[" <> pref <> "] "
   putBug bug
   where
-    prodVersion = T.unwords (bugVersion bug)
+    pref = T.unwords (bugProduct bug : bugVersion bug)
+
+bzUrl :: BugId -> String
+bzUrl bid = "https://" <> T.unpack brc <> "/show_bug.cgi?id=" <> show bid
 
 putBugId :: BugId -> IO ()
-putBugId bid =
-  putStrLn $ "https://" <> T.unpack brc <> "/show_bug.cgi?id=" <> show bid
+putBugId = putStrLn . bzUrl
 
 putBugURLStatus :: Bug -> IO ()
 putBugURLStatus bug = do
-  putStr $ "https://" <> T.unpack brc <> "/show_bug.cgi?id=" <> show (bugId bug)
+  putStr $ bzUrl $ bugId bug
   T.putStrLn $ " (" <> bugStatus bug <> ")"
 
 -- uniq for lists
@@ -475,7 +487,7 @@ getBzAccountId session muser = do
       else do
         users <- listBzUsers session userid
         case users of
-          [] -> error' $ "No user found for " ++ userid
+          [] -> error' $ "No user found for" +-+ userid
           [obj] -> return $ T.pack $ lookupKey' "email" obj
-          objs -> error' $ "Found multiple user matches: " ++
+          objs -> error' $ "Found multiple user matches:" +-+
                   unwords (map (lookupKey' "email") objs)
