@@ -5,6 +5,7 @@ module Cmd.SrcDeps (
 where
 
 import Distribution.RPM.Build.Graph
+import Safe (headMay)
 
 import Branches
 import Common
@@ -12,20 +13,20 @@ import Common.System
 import Git
 import Package
 
-srcDepsCmd :: Bool -> (Branch,[String]) -> IO ()
-srcDepsCmd rev (rbr,pkgs) =
-  srcDeps rev (rbr,pkgs) >>=
+srcDepsCmd :: Bool -> [String] -> (Branch,[String]) -> IO ()
+srcDepsCmd rev macros (rbr,pkgs) =
+  srcDeps rev macros (rbr,pkgs) >>=
   mapM_ (putStrLn . unwords)
 
-srcDeps :: Bool -> (Branch,[String]) -> IO [[String]]
-srcDeps rev (rbr,pkgs) = do
+srcDeps :: Bool -> [String] -> (Branch,[String]) -> IO [[String]]
+srcDeps rev macros (rbr,pkgs) = do
   when (null pkgs) $
     error' "please specify one or more package dirs"
   whenM isPkgGitRepo $
     error' "please run from the directory containing the dependency package set"
   listDirectory "." >>=
-    filterM checkPackage . filter ((/= '.') . head) >>=
-    fmap (topsortGraph Combine) . depsGraphDeps rev [] False [] [] False Nothing pkgs
+    filterM checkPackage . filter ((/= Just '.') . headMay) >>=
+    fmap (topsortGraph Combine) . depsGraphDeps rev (map ("-D" +-+) macros) False [] [] False Nothing pkgs
   where
     checkPackage :: FilePath -> IO Bool
     checkPackage p = do
