@@ -7,6 +7,8 @@ module ListReviews (
   listReviewsFull
   ) where
 
+import SimpleCmd (error')
+
 import Common
 
 import Branches
@@ -25,21 +27,35 @@ listReviews :: ReviewStatus -> IO [Bug]
 listReviews = listReviewsAll False
 
 listReviewsAll :: Bool -> ReviewStatus -> IO [Bug]
-listReviewsAll = listReviewsFull False Nothing Nothing
+listReviewsAll = listReviewsFull Nothing (Just Nothing) Nothing
 
-listReviewsFull :: Bool -> Maybe String -> Maybe String -> Bool
-                -> ReviewStatus-> IO [Bug]
-listReviewsFull assignee muser mpat allopen status = do
+-- FIXME: --unassigned
+listReviewsFull :: Maybe (Maybe String) -> Maybe (Maybe String)
+                -> Maybe String -> Bool -> ReviewStatus -> IO [Bug]
+listReviewsFull Nothing Nothing Nothing _ _ =
+  error' "please specify either report, assignee, or package name pattern"
+listReviewsFull mmassignee mmreporter mpat allopen status = do
   session <- bzApiKeySession
-  accountid <- getBzAccountId session muser
-  let reviews = (if assignee then assigneeIs else reporterIs) accountid .&&. maybe packageReview pkgReviewsPrefix mpat
+  massignedto <-
+    case mmassignee of
+      Nothing -> return Nothing
+      Just massignee ->
+        Just <$> getBzAccountId session massignee
+  mreporter <-
+    case mmreporter of
+      Nothing -> return Nothing
+      Just mreporter ->
+        Just <$> getBzAccountId session mreporter
+  let reviews = maybe id (\a -> (assigneeIs a .&&.)) massignedto $
+                maybe id (\r -> (reporterIs r .&&.)) mreporter $
+                maybe packageReview pkgReviewsPrefix mpat
       open = if allopen
         then statusOpen else
         case status of
           ReviewAllOpen -> statusOpen
           ReviewUnApproved -> statusOpen
           ReviewApproved -> statusNewPost
-          ReviewRepoCreated -> statusNewPost
+          ReviewRepoCreated -> statusRelPrep
           ReviewUnbranched -> statusNewModified
           ReviewBranched -> statusNewModified
           _ -> statusNewPost
@@ -74,4 +90,4 @@ listReviewsFull assignee muser mpat allopen status = do
     branched pkg = not <$> notBranched pkg
 
     notBranched :: String -> IO Bool
-    notBranched pkg = null <$> fedoraBranchesNoRawhide (pagurePkgBranches pkg)
+    notBranched pkg = null . delete Rawhide <$> listRemoteBranches pkg
