@@ -2,22 +2,30 @@
 
 module Main (main) where
 
+import Data.List.Extra (dropSuffix)
 import Data.Maybe (fromMaybe)
-import Distribution.Fedora.Branch
+import Distribution.Fedora.Branch (partitionBranches, readBranch)
 #if !MIN_VERSION_simple_cmd_args(0,1,7)
 import Options.Applicative (maybeReader, ReadM)
 #endif
+import SelectRPMs (existingStrategyOption, pkgMgrOpt, selectRpmsOptions,
+                   Yes(..))
 import SimpleCmdArgs
 
 -- commands
+import Cmd.Autospec
+import Cmd.BranchLogs
 import Cmd.Bugs
 import Cmd.Build
 import Cmd.Bump
 import Cmd.Clone
 import Cmd.Commit
-import Cmd.Compare
+import Cmd.CompareBranches
+import Cmd.CompareTarballs
+import Cmd.CreateReview
 import Cmd.Copr
 import Cmd.Diff
+import Cmd.Fetch
 import Cmd.FTBFS
 import Cmd.Import
 import Cmd.Install
@@ -29,12 +37,13 @@ import Cmd.Mock
 import Cmd.Override
 import Cmd.Owner
 import Cmd.Parallel
-import Cmd.PkgReview
 import Cmd.Prep
-import Cmd.PullPush
---import Cmd.Repoquery
+import Cmd.Pull
+import Cmd.Push
+import Cmd.Repoquery
 import Cmd.RequestBranch
 import Cmd.RequestRepo
+import Cmd.ReviewPackage
 import Cmd.Reviews
 import Cmd.Scratch
 import Cmd.SideTags
@@ -42,16 +51,20 @@ import Cmd.Sort
 import Cmd.SrcDeps
 import Cmd.Status
 import Cmd.Switch
+import Cmd.Unpushed
 import Cmd.Update
+import Cmd.UpdateReview
 import Cmd.WaitRepo
 
-import Bodhi (UpdateType(..),UpdateSeverity(..))
+import Bodhi (UpdateType(..), UpdateSeverity(..), UpdateNotes(..))
 import Branches
 import Common.System
 import Git (CommitOpt(..))
 import ListReviews
-import Package (ForceShort(..), BCond(..))
+import PkgReview
+import RpmBuild (ForceShort(..), BCond(..))
 import Paths_fbrnch (version)
+import Types (SideTagTarget(..))
 
 main :: IO ()
 main = do
@@ -61,10 +74,15 @@ main = do
     subcommands
     [ Subcommand "clone" "Clone packages" $
       cloneCmd
-      <$> cloneRequest
+      <$> dryrunOpt "Dry run"
+      <*> optional (optionWith branchM 'b' "branch" "BRANCH" "branch")
+      <*> cloneRequest
     , Subcommand "switch" "Switch branch" $
       switchCmd
-      <$> anyBranchArg
+      <$> verboseOpt "verbose output"
+      <*> switchLongWith "lenient" "keep going if package missing branch"
+      <*> switchLongWith "stash" "git stash local changes"
+      <*> anyBranchArg
       <*> manyPackages
     , Subcommand "nvr" "Print name-version-release" $
       nvrCmd
@@ -73,22 +91,38 @@ main = do
       statusCmd
       <$> switchWith 'n' "no-fetch" "Do not git fetch to save time"
       <*> switchWith 'r' "reviews" "Status of reviewed packages"
+      <*> switchWith 'l' "latest-commit" "Show the last commit"
+      <*> branchesPackages
+    , Subcommand "branch-log" "Show log with branches decor" $
+      branchLogCmd
+      <$> switchLongWith "latest" "Find related branches"
+      <*> switchLongWith "inactive" "Include old inactive branches"
+      -- FIXME currently actually no-op
+      <*> pure False --switchWith 'n' "no-simplify-by-decoration" "Do not filter for decorated commits"
       <*> branchesPackages
     , Subcommand "merge" "Merge from newer branch" $
       mergeCmd
-      <$> dryrunOpt
+      <$> dryrunOpt "Dry run (do not merge)"
+      <*> switchLongWith "no-fetch" "Skip git fetch"
       <*> nopromptOpt
       <*> optional (optionWith auto 's' "skip-bumps" "NUM" "Max num of rebuild commits to ignore [default 0]")
       <*> switchWith 'a' "show-all" "List all commits [default first 20]"
       <*> optional (optionWith branchM 'f' "from" "BRANCH" "Branch to merge from [default newer]")
       <*> branchesPackages
+    , Subcommand "unpushed" "Show unpushed commits" $
+      unpushedCmd
+      <$> switchWith 'c' "check-nvr" "Check NVR defined [default for < 10 pkgs]"
+      <*> switchWith 'l' "latest" "Only show latest unpushed commit"
+      <*> switchWith 'b' "bump" "Bump release (and commit) if no local commits"
+      <*> branchesPackages
     , Subcommand "build" "Build package(s) in Koji" $
       buildCmd
       <$> buildOpts
       <*> branchesPackages
-    , Subcommand "list" "List packages in pagure" $
+    , Subcommand "list" "List packages in pagure distgit" $
       listCmd
-      <$> switchWith 'c' "count" "Print number of packages"
+      <$> forceOpt "Do not prompt if large number of results"
+      <*> switchWith 'c' "count" "Print number of packages"
       <*> optional packagerOpt
       <*> many (pkgArg "PKGPAT...")
     , Subcommand "list-local" "List packages in branch" $
@@ -96,68 +130,79 @@ main = do
       <$> maybeBranchPackages True
     , Subcommand "branches" "List package branches" $
       branchesCmd
-      <$> switchWith 'd' "skip-dead" "Skip if dead.package exists"
+      <$> optional (flagLongWith' SkipDead "skip-dead" "Skip if dead.package exists" <|>
+                    flagLongWith' OnlyDead "only-dead" "Skip if not dead.package")
       <*> switchWith 'a' "all" "List all branches"
       <*> switchWith 'm' "missing" "Show missing branches"
       <*> branchesModeOpt
       <*> branchesPackages
     , Subcommand "parallel" "Parallel build packages in Koji" $
       parallelBuildCmd
-      <$> dryrunOpt
+      <$> dryrunOpt "Dry run: do not build anything"
       <*> mergeOpt
+      <*> optional (optionWith branchM 'f' "from" "BRANCH" "Branch to merge from [default newer]")
       <*> optionalWith auto 'l' "skip-to-layer" "LAYERNO" "Skip the first N layers [default 0]" 0
       <*> optional (sidetagTargetOpt $ Just "or creates one for you (with 'fedpkg request-side-tag --base-tag')")
+      <*> switchLongWith "must-push" "Error if no unpushed commits"
+      <*> optionalWith auto 'd' "delay" "SECONDS" "Sleep delay between starting builds [default 1s]" 1
       <*> updateOpt
-      <*> branchesPackages
+      <*> branchesPackagesDesc "BRANCH... PKG1... [:] PKG2..."
     , Subcommand "sidetags" "List user's side-tags" $
       sideTagsCmd
-      <$> switchLongWith "remove" "Remove one or more sidetags"
+      <$> dryrunOpt "Dry-run: no sidetag actions"
+      <*> optional (flagLongWith' SidetagTagged "tagged" "List tagged builds in sidetag(s)" <|>
+                    flagLongWith' SidetagAdd "create" "Create one or more sidetags" <|>
+                    flagLongWith' SidetagRemove "remove" "Remove one or more sidetags")
       <*> many branchArg
     , Subcommand "override" "Tag builds into buildroot override in Koji" $
       overrideCmd
-      <$> dryrunOpt
+      <$> dryrunOpt "Dry run: do not override"
+      <*> overrideModeOpt
       <*> optional (optionWith auto 'd' "duration" "DAYS" "Number of days until expiry [default 4]")
       <*> switchWith 'w' "no-wait" "Skip waitrepo step"
       <*> branchesPackages
     , Subcommand "waitrepo" "Wait for build to appear in Koji buildroot" $
       waitrepoCmd
-      <$> dryrunOpt
+      <$> dryrunOpt "Dry run: do not wait"
+      <*> switchLongWith "no-nvr" "Generic wait-repo without a build NVR"
       <*> waitfetchOpt
       <*> optional (sidetagTargetOpt Nothing)
       <*> branchesPackages
     , Subcommand "scratch" "Scratch build package in Koji" $
       scratchCmd
-      <$> dryrunOpt
+      <$> dryrunOpt "Dry run: do not build"
       <*> switchLongWith "stagger" "Stagger archs"
       <*> rebuildSrpmOpt
       <*> noFailFastOpt
+      <*> allowHeadOpt
       <*> optional archesOpt
       <*> many (sidetagTargetOpt Nothing)
-      <*> optional (strOptionWith 'r' "ref" "COMMITHASH" "git commit to build")
+      <*> optional scratchSourceOpt
       <*> branchesPackages
     , Subcommand "scratch-aarch64" "Koji aarch64 scratch build of package" $
       scratchCmdAarch64
-      <$> dryrunOpt
+      <$> dryrunOpt "Dry run: do not build"
       <*> rebuildSrpmOpt
+      <*> allowHeadOpt
       <*> switchWith 'X' "exclude-arch" "Exclude aarch64"
       <*> many (sidetagTargetOpt Nothing)
-      <*> optional (strOptionWith 'r' "ref" "COMMITHASH" "git commit to build")
+      <*> optional scratchSourceOpt
       <*> branchesPackages
     , Subcommand "scratch-x86_64" "Koji x86_64 scratch build of package" $
       scratchCmdX86_64
-      <$> dryrunOpt
+      <$> dryrunOpt "Dry run: do not build"
       <*> rebuildSrpmOpt
+      <*> allowHeadOpt
       <*> switchWith 'X' "exclude-arch" "Exclude x86_64"
       <*> many (sidetagTargetOpt Nothing)
-      <*> optional (strOptionWith 'r' "ref" "COMMITHASH" "git commit to build")
+      <*> optional scratchSourceOpt
       <*> branchesPackages
-    , Subcommand "update-version" "Update package in dist-git to newer version" $
-      updateCmd
-      <$> switchWith 's' "sources-only" "Only update sources"
-      <*> switchWith 'f' "force" "Download upstream sources even if they exist locally"
+    , Subcommand "update-sources" "Download and update newer sources" $
+      updateSourcesCmd
+      <$> forceOpt "Download upstream sources even if they exist locally"
       <*> switchWith 'H' "allow-head" "For updating inside rebase"
-      <*> maybeBranchPackages False
-    , Subcommand "sort" "Sort packages in build dependency order" $
+      <*> maybeBranchPackages' (Just "[VERSION]") False
+    , Subcommand "sort" "Sort packages in build dependency order (default format: chain-build)" $
       sortCmd
       <$> sortDisplayOpt
       <*> optional rpmWithOpt
@@ -165,127 +210,181 @@ main = do
     , Subcommand "prep" "Prep sources" $
       prepCmd
       <$> optional prepPreOpts
-      <*> switchWith 'v' "verbose" "show rpmbuild output"
+      <*> verboseOpt "show rpmbuild output"
+      <*> switchWith 'd' "deps" "require deps to be installed"
+      <*> allowHeadOpt
       <*> maybeBranchPackages False
     , Subcommand "local" "Build locally" $
       localCmd
-      <$> switchWith 'q' "quiet" "Hide the build.log until it errors"
-      <*> switchWith 'd' "debug" "show the rpmbuild command"
+      <$> quietOpt "Hide the build.log until it errors"
+      <*> debugOpt "show the rpmbuild command"
+      <*> allowHeadOpt
+      <*> jobsOpt
       <*> optional forceshortOpt
       <*> many bcondOpt
       <*> branchesPackages
     , Subcommand "srpm" "Build srpm" $
       srpmCmd
-      <$> switchWith 'f' "force" "regenerate even if spec older than existing srpm"
+      <$> forceOpt "regenerate even if spec older than existing srpm"
       <*> maybeBranchPackages False
+    , Subcommand "srpm-spec" "Show the spec file in an srpm" $
+      srpmSpecCmd
+      <$> switchWith 'd' "diff" "Compare with current spec file"
+      <*> some (strArg "SRPM")
     , Subcommand "diff" "Diff local changes" $
       diffCmd
-      <$> switchWith 'o' "spec-only" "Only diff spec file"
+      <$> debugOpt "use package headers"
+      <*> switchWith 'o' "spec-only" "Only diff spec file"
       <*> diffWorkOpt
       <*> diffFormatOpt
       <*> switchWith 'i' "ignore-bumps" "Ignore pure release bumps"
-      <*> optional diffFilterOpt
-      <*> optional (optionWith anyBranchM 'w' "with-branch" "BRANCH" "branch")
+      <*> many diffFilterOpt
+      <*> optional (optionWith anyBranchM 'w' "with-branch" "BRANCH" "Compare with BRANCH")
       <*> maybeBranchPackages False
-    , Subcommand "compare" "Show commits between branches" $
-      compareCmd
+    , Subcommand "compare-branches" "Show commits between branches" $
+      compareBranchesCmd
       <$> switchWith 'l' "long" "show full commit log"
       <*> optional (strOptionWith 'i' "ignore" "SUBSTRING" "Matching substring to ignore")
       <*> anyBranchArg
       <*> anyBranchArg
       <*> manyPackages
+    , Subcommand "compare-tarballs" "Diff source tarballs" $
+      compareTarballsCmd
+      <$> optional (optionLongWith auto "max-line-length" "LENGTH" "Skip diff files longer than this [default 200]")
+      <*> optional (strArg "[NAME-]VERSION1")
+      <*> optional (strArg "[NAME-]VERSION2")
     , Subcommand "src-deps" "List source package dependencies" $
       srcDepsCmd
       <$> switchWith 'r' "reverse" "Reverse dependencies"
+      <*> many (strOptionWith 'D' "define" "'MACRO DEF'" "Define RPM macro")
       <*> branchPackages
     , Subcommand "mock" "Local mock build" $
       mockCmd
-      <$> switchWith 'n' "dry-run" "Do not build (but creates srpm)"
+      <$> dryrunOpt "Do not build (but creates srpm)"
       <*> optional nocleanOpt
-      <*> switchWith 'w' "network" "Use network during build"
-      <*> switchWith 's' "shell" "Enter chroot shell after building"
+      <*> switchWith 'N' "network" "Use network during build"
+      <*> switchLongWith "reinstall" "Allow reinstallation with --install"
+      <*> optional (flagLongWith' ShellOnly "shell-only" "Skip mock build" <|>
+                    flagWith' BuildShell 's' "shell" "Enter chroot shell after building" <|>
+                    flagLongWith' MockInstall "install" "Install the built rpms")
       <*> optional (optionWith branchM 'r' "root" "BRANCH" "Mock config to use")
+      <*> optional archOpt
       <*> branchesPackages
-    , Subcommand "install-deps" "Install package build dependencies" $
+      -- was install-deps
+    , Subcommand "builddeps" "Install package build dependencies" $
       installDepsCmd
       <$> maybeBranchPackages False
     , Subcommand "install" "Build locally and install package(s)" $
       -- FIXME drop --shortcircuit from install?
       installCmd
-      <$> switchWith 'v' "verbose" "verbose rpmbuild output"
+      <$> quietOpt "Quiet rpmbuild output"
       <*> switchWith 'R' "recurse" "build and install missing deps packages"
       <*> optional (optionLongWith branchM "from" "BRANCH" "Merge branch first")
+      <*> jobsOpt
       <*> optional forceshortOpt
       <*> many bcondOpt
+      <*> optional pkgMgrOpt
       <*> switchWith 'r' "reinstall" "reinstall rpms"
-      <*> switchWith 'a' "all-subpackages" "install all subpackages (default if none currently installed)"
+      <*> switchLongWith "allow-erasing" "use dnf --allowerasing"
+      <*> switchLongWith "no-build" "do not (re-)build (install built rpms)"
+      <*> switchLongWith "ignore-builddeps" "do not install builddeps"
+      <*> flagWith No Yes 'y' "yes" "Assume yes to questions"
+      <*> selectRpmsOptions
+      <*> optional existingStrategyOption
       <*> maybeBranchPackages False
     , Subcommand "not-installed" "Packages not installed locally" $
       notInstalledCmd
       <$> maybeBranchPackages False
+    , Subcommand "uninstall" "Remove installed package(s)" $
+      unInstallCmd
+      <$> flagWith No Yes 'y' "yes" "Assume yes to questions"
+      <*> maybeBranchPackages False
     , Subcommand "bugs" "List package bugs" $
       bugsCmd
       <$> optional (strOptionWith 's' "summary" "KEY" "Search for bugs containing keyword")
       <*> manyPackages
     , Subcommand "bump" "Bump release for package" $
-      bumpPkgs
-      <$> switchWith 'l' "local" "Use local origin rather than checking latest koji"
-      <*> optional commitOpts
+      bumpCmd
+      <$> dryrunOpt "Dry run: do not bump"
+      <*> switchLongWith "strict" "Check koji if NVR was built"
+      <*> optional (strOptionWith 'm' "message" "COMMITMSG"
+                    "Specify commit message")
+      <*> optional (strOptionWith 'c' "changelog" "CLOGENTRY"
+                    "Override changelog entry [default: rebuild]")
       <*> branchesPackages
     , Subcommand "commit" "Git commit packages" $
-      commitPkgs
-      <$> optional commitOpts
+      commitCmd
+      <$> dryrunOpt "Dry run: do not commit"
+      <*> switchWith 'b' "branched" "Commit to branched dist-git"
+      <*> optional commitOpts
       <*> switchWith '1' "first-line" "use first line of changelog"
       <*> switchWith 'a' "unstaged" "include unstaged changes"
       <*> manyPackages
     , Subcommand "pull" "Git pull packages" $
       pullPkgs
-      <$> pullOpts
+      <$> optional pullOpts
       <*> branchesPackages
     , Subcommand "fetch" "Git fetch packages" $
       fetchPkgs
-      <$> manyPackages
+      <$> switchLongWith "lenient" "skip non dist-git directories"
+      <*> manyPackages
     , Subcommand "push" "Git push packages" $
       pushPkgs
-      <$> branchesPackages
+      <$> dryrunOpt "Dry run: do not push"
+      <*> switchLongWith "no-fetch" "Skip git fetch"
+      <*> optional (strOptionWith 'r' "ref" "COMMITHASH" "git commit to push")
+      <*> branchesPackages
     , Subcommand "owner" "List package owner(s)" $
       ownerCmd
       <$> manyPackages
+    , Subcommand "bzusers" "Search bugzilla users" $
+      bzusersCmd
+      <$> strArg "NAME"
     , Subcommand "create-review" "Create a Package Review request" $
-      createReview
-      <$> scratchOpt
+      createReviewCmd
+      <$> forceOpt "create a review even if one exists already"
+      <*> optional scratchOpt
       <*> mockOpt False
       <*> manyPackages
     , Subcommand "update-review" "Update a Package Review" $
-      updateReview
-      <$> scratchOpt
+      updateReviewCmd
+      <$> optional scratchOpt
       <*> mockOpt False
       <*> optional (strArg "SPECFILE")
     , Subcommand "review-package" "Run fedora-review on a package Review Request bug" $
       reviewPackage
-      <$> optional (pkgArg "PACKAGE/BZID")
+      <$> switchWith 'f' "full" "Download and check package review with fedora-review mock build"
+      <*> optional (pkgArg "PACKAGE/BZID")
     , Subcommand "reviews" "List package reviews" $
       reviewsCmd
       <$> reviewShortOpt
       <*> reviewAllStatusOpt
-      <*> switchWith 'T' "assigned-to" "List reviews assigned to user"
-      <*> optional (strOptionWith 'U' "user" "USER" "Bugzilla user email")
+      <*> optional
+      (flagLongWith' Nothing "assigned" "Package reviewer" <|>
+       Just <$> strOptionLongWith "assignee" "BZUSER" "Package review submitter")
+      <*> optional
+      (flagLongWith' Nothing "submitted" "Submitted reviews [default if no assignee specified]" <|>
+       Just <$> strOptionWith 's' "submitter" "BZUSER" "Package review submitter")
       <*> optional (strOptionWith 'p' "pattern" "PKGPREFIX" "Package pattern prefix")
       <*> reviewStatusOpt
     , Subcommand "request-repos" "Request dist git repo for new approved packages" $
       requestRepos
       <$> mockOpt True
       <*> reviewAllStatusOpt
-      <*> switchWith 'r' "retry" "Re-request repo"
+      <*> switchWith 's' "skip-request-check" "Skip check for existing fedora-scm-requests issue"
+      <*> switchWith 'r' "resubmit" "Re-request repo"
       <*> branchesPackages
     , Subcommand "import" "Import new approved created packages from bugzilla review" $
-      importCmd
-      <$> mockOpt True
+      importCmd True False
+      <$> switchLongWith "existing" "Use an existing repo"
+      <*> mockOpt True
       <*> branchesPackages
     , Subcommand "request-branches" "Request branches for approved created packages" $
       requestBranchesCmd
-      <$> switchWith 'q' "quiet" "Quieter output"
-      <*> optional (optionWith branchM 'r' "recurse-from" "BRANCH" "Add neighboring dependencies from branch")
+      <$> quietOpt "Quieter output"
+      <*> switchWith 'r' "reviews" "Request branches for package reviews"
+      <*> switchLongWith "skip-check" "Skip koji check for branch package"
+      <*> optional (optionWith branchM 'R' "recurse-from" "BRANCH" "Add neighboring dependencies from branch")
       <*> mockOpt False
       <*> branchesPackages
     , Subcommand "find-review" "Find package review bug" $
@@ -302,14 +401,21 @@ main = do
       <*> branchesPackages
     , Subcommand "copr" "Build package(s) in Fedora Copr" $
       coprCmd
-      <$> dryrunOpt
-      <*> switchWith 'l' "list-chroots" "Show project chroots"
-      <*> buildByOpt
+      <$> dryrunOpt "Dry run: do not build"
+      <*> (flagWith' ListChroots 'l' "list-chroots" "Show project chroots" <|>
+           flagLongWith' CoprNew "new" "Create new copr repo" <|>
+           -- FIXME: NonEmpty String
+           flagWith' CoprMonitor 'm' "monitor" "Show project chroots" <*> optional (strOptionLongWith "filter" "NEEDLE" "Filter to results matching NEEDLE") <*> switchLongWith "name-only" "Only print package names" <|>
+           flagLongWith' CoprFailures "failures" "Show build failures with build.log tails" <*> optional (strOptionLongWith "filter" "NEEDLE" "Filter to results matching NEEDLE") <|>
+           -- FIXME: NonEmpty String
+           pure CoprBuild)
+      <*> forceOpt "build even if existing n-v-r"
+      <*> optional buildByOpt
       <*> optional archesOpt
       <*> pkgArg "PROJECT"
       <*> branchesPackages
     , Subcommand "rename-rawhide" "Rename local 'master' branch to 'rawhide'" $
-      renameMasterCmd
+      renameRawhideCmd
       <$> manyPackages
     , Subcommand "count" "Count number of living packages" $
       countCmd
@@ -318,12 +424,12 @@ main = do
       graphCmd
       <$> switchWith 'o' "output" "Output graph in gv/dot format"
       <*> optional rpmWithOpt <*> maybeBranchPackages True
-    -- , Subcommand "repoquery" "Repoquery branches (put repoquery options after '--')" $
-    --   repoqueryCmd
-    --   <$> branchesPackages
-    , Subcommand "ftbfs" "Check FTBFS status" $
+    , Subcommand "repoquery" "Repoquery branches (put repoquery options after '--')" $
+      repoqueryCmd
+      <$> branchesPackages
+    , Subcommand "ftbfs" "Check FTBFS bug status" $
       ftbfsCmd
-      <$> dryrunOpt
+      <$> dryrunOpt "dry run"
       <*> switchWith 'l' "short" "Only list packages"
       <*> optional (flagWith' (FtbfsUser Nothing) 'M' "mine" "Your packages"
                     <|>
@@ -331,16 +437,34 @@ main = do
                     <|>
                     FtbfsSubstring <$> strOptionWith 's' "substring" "STRING" "Component substring")
       <*> maybeBranchPackages False
+    , Subcommand "autospec" "Convert package to use rpmautospec" $
+      autospecCmd
+      <$> forceOpt "Refresh changelog file to current"
+      <*> manyPackages
+    , Subcommand "unautospec" "Unconvert rpmautospec package" $
+      unautospecCmd
+      <$> branchesPackages
+    , Subcommand "move-artifacts" "Move old rpm artifacts into rpmbuild dirs" $
+      moveArtifactsCmd
+      <$> switchWith 'd' "delete" "Remove duplicate artifacts"
+      <*> manyPackages
+    , Subcommand "tag-build-to-sidetag" "Tag NVR for current branch to sidetag" $
+      tagBuildToSidetagCmd
+      <$> dryrunOpt "Dry run"
+      <*> switchWith 'k' "allow-dirty" "Allow unclean git repo"
+      <*> optionWith branchM 't' "to" "BRANCH" "Target branch of sidetag"
+      <*> maybeBranchPackages False
     ]
   where
     cloneRequest :: Parser CloneRequest
     cloneRequest =
       flagWith' (CloneUser Nothing) 'M' "mine" "Your packages" <|>
       CloneUser . Just <$> strOptionWith 'u' "user" "USER" "Packages of FAS user" <|>
-      ClonePkgs <$> maybeBranchPackages True
+      CloneGroup <$> strOptionWith 'g' "group" "GROUP" "Packages of package group" <|>
+      ClonePkgs <$> somePackages
 
     reviewShortOpt :: Parser Bool
-    reviewShortOpt = switchWith 's' "short" "Only output the package name"
+    reviewShortOpt = switchLongWith "short" "Only output the package name"
 
     reviewAllStatusOpt :: Parser Bool
     reviewAllStatusOpt = switchWith 'A' "all-status" "include all open states"
@@ -371,13 +495,13 @@ main = do
     anyBranchM = anyBranch <$> str
 
     pkgArg :: String -> Parser String
-    pkgArg lbl = removeSuffix "/" <$> strArg lbl
+    pkgArg lbl = dropSuffix "/" <$> strArg lbl
 
     manyPackages :: Parser [String]
     manyPackages =  many (pkgArg "PKGPATH...")
 
-    -- somePackages :: Parser [String]
-    -- somePackages = some (pkgArg "PKGPATH...")
+    somePackages :: Parser [String]
+    somePackages = some (pkgArg "PKG...")
 
     branchesOpt :: Parser (Maybe BranchOpts)
     branchesOpt =
@@ -404,14 +528,18 @@ main = do
           in case brs of
             [] -> error' "specify one branch"
             [br] -> (br,pkgs)
-            _ -> error' $ "cannot have more than one branch: " ++ unwords (map show brs)
+            _ -> error' $ "cannot have more than one branch:" +-+ unwords (map showBranch brs)
 
     maybeBranchPackages :: Bool -> Parser (Maybe Branch,[String])
-    maybeBranchPackages oneplus =
+    maybeBranchPackages = maybeBranchPackages' Nothing
+
+    maybeBranchPackages' :: Maybe String -> Bool
+                        -> Parser (Maybe Branch,[String])
+    maybeBranchPackages' mextraarg oneplus =
       maybeBranchesPkgs <$>
       if oneplus
-      then some (pkgArg "[BRANCH] PKGPATH...")
-      else many (pkgArg "[BRANCH] [PKGPATH]...")
+      then some (pkgArg $ "[BRANCH]" +-+ extraargdesc +-+ "PKGPATH...")
+      else many (pkgArg $ "[BRANCH]" +-+ extraargdesc +-+ "[PKGPATH]...")
       where
         maybeBranchesPkgs :: [String] -> (Maybe Branch,[String])
         maybeBranchesPkgs args =
@@ -419,12 +547,17 @@ main = do
           in case brs of
             [] -> (Nothing, pkgs)
             [br] -> (Just br,pkgs)
-            _ -> error' $ "cannot have more than one branch: " ++ unwords (map show brs)
+            _ -> error' $ "cannot have more than one branch:" +-+ unwords (map showBranch brs)
+
+        extraargdesc = fromMaybe "" mextraarg
+
+    branchesPackages :: Parser (BranchesReq, [String])
+    branchesPackages = branchesPackagesDesc "BRANCH... PKGPATH..."
 
     -- FIXME split later to prevent branches after packages
-    branchesPackages :: Parser (BranchesReq, [String])
-    branchesPackages =
-      branchesReqPkgs <$> branchesOpt <*> many (pkgArg "BRANCH... PKGPATH...")
+    branchesPackagesDesc :: String -> Parser (BranchesReq, [String])
+    branchesPackagesDesc desc =
+      branchesReqPkgs <$> branchesOpt <*> many (pkgArg desc)
       where
         branchesReqPkgs :: Maybe BranchOpts -> [String] -> (BranchesReq, [String])
         branchesReqPkgs mbrnchopts args =
@@ -447,7 +580,7 @@ main = do
     mockOpt brs = switchWith 'm' "mock" $ "Do mock build to test" ++ if brs then " branches" else ""
 
     archOpt :: Parser String
-    archOpt = strOptionWith 'a' "arch" "ARCH[,ARCH].." "build for arch(s)"
+    archOpt = strOptionWith 'a' "arch" "ARCH" "build for arch(s)"
 
     rebuildSrpmOpt = switchWith 'S' "rebuild-srpm" "rebuild srpm in Koji"
 
@@ -458,24 +591,28 @@ main = do
     buildOpts =
       BuildOpts
       <$> mergeOpt
+      <*> optional (optionWith branchM 'f' "from" "BRANCH" "Branch to merge from [default newer]")
       <*> noFailFastOpt
       <*> optional (sidetagTargetOpt Nothing)
       <*> overrideOpt
       <*> waitrepoOpt
-      <*> dryrunOpt
+      <*> dryrunOpt "Dry run: do not merge/push/build"
       <*> skipFetchOpt
+      <*> gitrefOpt
       <*> updateOpt
-      <*> useChangelogOpt
+      <*> optional notesOpt
       <*> switchWith 'p' "by-package" "Build by each package across brs"
-      <*> switchWith 'k' "allow-dirty" "Allow building from unclean git dir"
+      <*> switchLongWith "stash" "git stash before fetching and building"
       where
         overrideOpt =
           optional (optionWith auto 'o' "override" "DAYS" "Create buildroot override for specified days: implies --wait-repo")
         waitrepoOpt =
           optional (flagWith' True 'w' "waitrepo" "Waitrepo for each build" <|>
                     flagWith' False 'W' "no-waitrepo" "Do not waitrepo for each build")
-        useChangelogOpt =
-          switchWith 'c' "changelog-notes" "Use spec changelog for Bodhi notes"
+        gitrefOpt = optional (strOptionLongWith "ref" "COMMIT" "commit to build")
+        notesOpt =
+          flagWith' NotesChangelog 'c' "changelog-notes" "Use spec changelog for Bodhi notes" <|>
+          NotesText <$> strOptionLongWith "notes" "NOTES" "Bodhi update notes"
 
 --    yesOpt = switchWith 'y' "yes" "Assume yes for questions"
 
@@ -485,12 +622,15 @@ main = do
     noFailFastOpt = switchWith 'f' "no-fail-fast" "Do not --fail-fast"
 
     excludeArch :: Parser String
-    excludeArch = strOptionWith 'X' "exclude-arch" "ARCH[,ARCH].." "build without arch(s)"
+    excludeArch = strOptionWith 'X' "exclude-arch" "ARCH" "build without arch"
 
     archesOpt :: Parser Archs
     archesOpt = Archs <$> some archOpt <|> ExcludedArchs <$> some excludeArch
 
-    dryrunOpt = switchWith 'n' "dry-run" "Do not write (push, build, post, override)"
+    dryrunOpt desc = switchWith 'n' "dry-run" desc <|>
+                     switchLongWith "dryrun" "alias for --dry-run"
+
+    debugOpt = switchWith 'd' "debug"
 
     skipFetchOpt = switchWith 'S' "skip-fetch" "Do not git fetch"
 
@@ -516,7 +656,7 @@ main = do
 
     forceshortOpt =
       flagWith' ForceBuild 'f' "rebuild" "Rebuild even if already built" <|>
-      flagLongWith' ShortCompile "short-compile" "Do rpmbuild --short-circuit -bc" <|>
+      flagWith' ShortCompile 'c' "short-compile" "Do rpmbuild --short-circuit -bc" <|>
       flagWith' ShortInstall 's' "short-install" "Do rpmbuild --short-circuit -bi"
 
     diffFormatOpt :: Parser DiffFormat
@@ -542,22 +682,22 @@ main = do
       strOptionWith 'm' "message" "COMMITMSG" "commit message" <|>
       flagWith' CommitAmend 'A' "amend" "Amend commit"
 
-    pullOpts :: Parser PullOpts
+    pullOpts :: Parser PullOpt
     pullOpts =
-      PullOpts <$>
-      switchWith 'l' "lenient" "Ignore non-git dirs and files" <*>
-      switchWith 'f' "no-fetch" "Skip git fetch"
+      flagWith' PullLenient 'l' "lenient" "Ignore non-git dirs and files" <|>
+      flagWith' PullNoFetch 'f' "no-fetch" "Skip git fetch" <|>
+      flagWith' PullStash 's' "stash" "Stash local changes" <|>
+      flagWith' PullRebase 'r' "rebase" "Git pull instead of fetch"
 
-    buildByOpt = flagWith' SingleBuild 'S' "single" "Non-progressive normal single build" <|> flagWith' BuildByRelease 'R' "by-release" "Builds by release" <|> flagWith ValidateByRelease ValidateByArch 'A' "by-arch" "Build across latest release archs first (default is across releases for primary arch)"
+    buildByOpt =
+      flagWith' SingleBuild 'S' "single" "Non-progressive normal single build" <|>
+      flagWith' BuildByRelease 'R' "by-release" "Builds by release" <|>
+      flagWith' ValidateByArch 'A' "by-arch" "Build across latest release archs first (default is across releases for primary arch)"
 
     commandOpt = strOptionWith 'c' "cmd" "SHELLCOMMAND" "Shell command to run in $p"
 
     targetOpt :: Parser String
-    targetOpt =
-      checkNotRawhide <$> strOptionWith 't' "target" "TARGET" "Koji target"
-      where
-        checkNotRawhide "rawhide" = error' "'rawhide' is not a valid target!"
-        checkNotRawhide t = t
+    targetOpt = strOptionWith 't' "target" "TARGET" "Koji target"
 
     sidetagTargetOpt :: Maybe String -> Parser SideTagTarget
     sidetagTargetOpt mdesc =
@@ -577,8 +717,14 @@ main = do
 
     scratchOpt :: Parser ScratchOption
     scratchOpt =
-      ScratchTask <$> optionWith auto 's' "scratch-build" "TASKID" "Existing scratch build taskid" <|>
-      flagWith ScratchBuild SkipScratch 'S' "no-scratch" "Skip scratch build"
+      ScratchTask <$> optionLongWith auto "scratch-task" "TASKID" "Existing scratch build taskid" <|>
+      flagWith' SkipScratch 'S' "no-scratch" "Skip koji scratch build" <|>
+      flagWith' ScratchBuild 's' "scratch" "Do koji scratch build without prompt"
+
+    scratchSourceOpt :: Parser ScratchSource
+    scratchSourceOpt =
+      ScratchSRPM <$> strOptionLongWith "srpm" "SRPM" "SRPM to use" <|>
+      ScratchRef <$> strOptionWith 'r' "ref" "COMMITHASH" "git commit to build"
 
     nocleanOpt :: Parser NoClean
     nocleanOpt =
@@ -590,11 +736,28 @@ main = do
     sortDisplayOpt :: Parser SortDisplay
     sortDisplayOpt =
       flagWith' SortParallel 'p' "parallel" "Group dependent packages on separate lines"
-      <|> flagWith' SortChain 'c' "chain" "chain-build output"
-      <|> flagWith SortPlain SortLayers 'l' "layers" "output parallel layers"
+      <|> flagWith' SortLayers 'l' "layers" "output parallel layers"
+      <|> flagWith SortChain SortPlain 's' "separated" "Dependent grouping separated by empty lines"
 
     -- for waitrepo
     waitfetchOpt :: Parser WaitFetch
     waitfetchOpt =
       flagWith' WaitDirty 'k' "allow-dirty" "Allow unclean git repo" <|>
       flagWith WaitFetch WaitNoFetch 'F' "no-fetch" "Skip git fetch"
+
+    overrideModeOpt :: Parser OverrideMode
+    overrideModeOpt =
+      flagWith' OverrideList 'l' "list" "List active override(s)" <|>
+      flagWith OverrideCreate OverrideExpire 'X' "expire" "Expire override(s)"
+
+    verboseOpt :: String -> Parser Bool
+    verboseOpt = switchWith 'v' "verbose"
+
+    quietOpt :: String -> Parser Bool
+    quietOpt = switchWith 'q' "quiet"
+
+    allowHeadOpt = switchLongWith "detached-head" "allow detached HEAD"
+
+    forceOpt = switchWith 'f' "force"
+
+    jobsOpt = optional (optionWith auto 'j' "jobs" "NUM" "Max processes in rpmbuild")
