@@ -1,6 +1,8 @@
 module Pagure (
   srcfpo,
   pagureio,
+  pagureGroupInfo,
+  pagureGroupRepos,
   pagureListGitBranches,
   pagureListProjectIssueTitlesStatus,
   IssueTitleStatus(..),
@@ -10,7 +12,7 @@ module Pagure (
   printScmIssue
   ) where
 
-import Common ((+/+))
+import Common ((+/+), (+-+))
 import qualified Common.Text as T
 
 import Fedora.Pagure
@@ -23,6 +25,6 @@ pagureio = "pagure.io"
 
 printScmIssue :: IssueTitleStatus -> IO ()
 printScmIssue issue =
-  putStrLn $ "https://" ++ pagureio +/+ "releng/fedora-scm-requests" +/+ "issue" +/+ show (pagureIssueId issue) ++ " (" ++ T.unpack (pagureIssueStatus issue) ++ mclosed ++ "): " ++ pagureIssueTitle issue
+  putStrLn $ "https://" ++ pagureio +/+ "releng/fedora-scm-requests" +/+ "issue" +/+ show (pagureIssueId issue) +-+ "(" ++ T.unpack (pagureIssueStatus issue) ++ mclosed ++ "):" +-+ pagureIssueTitle issue
   where
     mclosed = maybe "" (\s-> ":" ++ T.unpack s) $ pagureIssueCloseStatus issue
